//
//  DEMOView.m
//  HostDisplayInfo
//
//  Created by Dave J. Vrba on 1/19/10.
//  Copyright 2010 __MyCompanyName__. All rights reserved.
//

#import "DEMOView.h"


@implementation DEMOView

- (id)initWithFrame:(NSRect)frame
{
    self = [super initWithFrame:frame];
    if( self)
    {
        // Initialization code here.
        _grad_components[0] = 0.1;
        _grad_components[1] = 0.6;
        _grad_components[2] = 0.1;
        _grad_components[3] = 0.9;

        _grad_components[4] = 0.2;
        _grad_components[5] = 0.1;
        _grad_components[6] = 1.0;
        _grad_components[7] = 0.1;
        
        _ignore_rect = NSZeroRect;
    }
    
    return self;
}

- (void)drawRect:(NSRect)rect
{
    // context of view (current)
    CGContextRef cgcontext = (CGContextRef)[[NSGraphicsContext currentContext] graphicsPort];

    // my visible rect
    NSRect my_rect = [self visibleRect];

    // draw into the context
    CGFloat components[4] = { 0.3, 0.1, 1.0, 0.4};
    CGColorRef cgcolor = CGColorCreate( CGColorSpaceCreateDeviceRGB(), components);
    CGContextSetFillColorWithColor( cgcontext, cgcolor);
    CGContextFillRect( cgcontext, NSRectToCGRect(rect));
    CFRelease( cgcolor);

    CGFloat grad_locations[] =
    {
        0.0, 1.0
    };
    CGGradientRef gradient = CGGradientCreateWithColorComponents( CGColorSpaceCreateDeviceRGB(), _grad_components, grad_locations, 2);
    CGPoint grad_start = CGPointMake( 0.0, 0.0);
    CGPoint grad_end = CGPointMake( my_rect.size.width, my_rect.size.height);
    CGContextDrawLinearGradient( cgcontext, gradient, grad_start, grad_end, 0);
    CGGradientRelease( gradient);


    CGColorRef white_color = CGColorCreateGenericRGB( 1.0, 1.0, 1.0, 0.8);
    CGContextSelectFont( cgcontext, "Helvetica", 16.0, kCGEncodingMacRoman);
    CGContextSetFillColorWithColor( cgcontext, white_color);
    CGContextSetTextDrawingMode( cgcontext, kCGTextFill);
    CGAffineTransform text_matrix = CGAffineTransformConcat( CGAffineTransformMakeRotation( M_PI_4/2.0), CGAffineTransformMakeTranslation( 10.0, 10.0));
    CGContextSetTextMatrix( cgcontext, text_matrix);
    const char *my_string = "Left mouse click and drag mouse inside this gradient";
    CGContextShowText( cgcontext, my_string, strlen( my_string));
}

- (BOOL)canBecomeKeyView
{
    return YES;
}

- (BOOL)acceptsFirstResponder
{
    return YES;
}

- (BOOL)becomeFirstResponder
{
    return YES;
}

- (BOOL)resignFirstResponder
{
    return YES;
}

- (BOOL)acceptsFirstMouse:(NSEvent *)theEvent
{
    return YES;
}

- (BOOL)preservesContentDuringLiveResize
{
    return NO;
}

- (BOOL)isFlipped
{
    return NO;
}

- (BOOL)isOpaque
{
	return NO;
}

- (BOOL)wantsDefaultClipping
{
//    return NO; // This means we must keep our drawing constrained to the regions of the window being drawn.
    return YES;
}

- (void)setIgnoreFrame:(NSRect)frame
{    
    _ignore_rect = frame;
}

- (void)mouseDown:(NSEvent *)theEvent
{
    // pass mouse to next responder if mouse is inside the ignore rectangle
    NSPoint mouse_local = [self convertPoint:[theEvent locationInWindow] fromView:nil];
    if( NSMouseInRect( mouse_local, _ignore_rect, NO))
    {
        [[self nextResponder] mouseDown:theEvent];
    }
}

- (void)mouseDragged:(NSEvent *)theEvent
{
    NSRect my_rect = [self visibleRect];

    NSPoint mouse_local = [self convertPoint:[theEvent locationInWindow] fromView:nil];
    mouse_local.x /= my_rect.size.width;
    mouse_local.y /= my_rect.size.height;
    _grad_components[0] = 1.0 * mouse_local.x;
    _grad_components[6] = 1.0 * mouse_local.y;

    [self setNeedsDisplay:TRUE];
}

- (void)resetCursorRects
{
    [self addCursorRect:[self visibleRect] cursor:[NSCursor crosshairCursor]];
}


@end
